(function(){

const K = window.K;

K.NateAbstractDom = class K_NateAbstractDom
{
  constructor(parentOrElem)
  {
    this._init(parentOrElem);
  }

  _init(parentOrElem)
  {
    this.children = [];
    this.nateParent = null;
    this.elem = null;
    this.isDeleted = false;

    // If there is parentOrElem, check the type of it.
    if (K.Object.isNotNull(parentOrElem))
    {
      if (parentOrElem instanceof K.NateAbstractDom)
      {
        this.nateParent = parentOrElem;
        throw 'Nate parent is not properly handled in constructor yet !'
      }
      else if (K.Elem.isElem(parentOrElem) || K.Elem.isText(parentOrElem))
      {
        if (K.Object.isNotNull(parentOrElem.nate))
        {
          throw 'Cannot create 2nd Nate object for same elem! The element you provided as elem for Nate is already a elem for different nate object';
        }
        else
        {
          // First, attach this Nate object to element.
          this.attachNateToElem(parentOrElem);

          // Then, check if DOM parent is Nate object - if yes, tell it about me.
          if (K.Object.isNotNull(parentOrElem.parentNode) &&
              K.Object.isNotNull(parentOrElem.parentNode.nate))
          {
            parentOrElem.parentNode.nate.addNateChild(this);
          }
        }
      }
      else
      {
        throw 'parentOrElem should be Nate object or nate-element';
      }
    }
    else
    {
      throw 'parentOrElem should be Nate object or nate-element - cannot be null';
    }
  }

  _confirmIsNotDeleted()
  {
    if (this.isDeleted)
    {
      console.log('PANIC! Nate item', this.elem, 'already deleted');

      throw new K.Error({
        errorCode: 'nateItemAlreadyDeleted',
        ext: {elem: this.elem}});
    }
  }

  deleteAllChildren()
  {
    this._confirmIsNotDeleted();
    // Possible improvement: optimize this
    while (this.children.length > 0)
    {
      this.children[0].delete();
    }
  }

  delete()
  {
    this._confirmIsNotDeleted();

    if (K.Object.isNotNull(this.nateParent))
    {
      this.isDeleted = true;
      this.nateParent.removeNateChild(this);
    }
    else
    {
      this.elem.style.backgroundColor = '#f00000';
      throw 'Removal from non-nates is NOT supported. You tried to remove [elem:' + this.elem + ']';
    }
  }

  addNateChild(nateChild)
  {
    if (K.Object.isNotNull(nateChild.nateParent))
    {
      throw 'Moving nate-child between parents not yet supported.';
    }

    nateChild.nateParent = this;
    this.children.push(nateChild);

    if (K.Object.isNotNull(this.elem) &&
        K.Object.isNotNull(nateChild.elem) &&
        (nateChild.elem.parentNode != this.elem))
    {
      this.elem.appendChild(nateChild.elem);
    }
  }

  removeNateChild(nateChild)
  {
    if (nateChild.nateParent != this)
    {
      throw 'Can\'t remove nateChild from me, as the child claims his parent is another object.';
    }

    K.Array.removeAll(this.children, nateChild);

    if (K.Object.isNotNull(this.elem) &&
        K.Object.isNotNull(nateChild.elem))
    {
      try
      {
        this.elem.removeChild(nateChild.elem);
      }
      catch(err)
      {
        console.log('Error during removeChild parent is:', this.elem, 'child is:', nateChild.elem);
      }
    }

    nateChild.nateParent = null;
  }

  attachNateToElem(elem)
  {
    this._confirmIsNotDeleted();

    // Avoid registering >1 Nate objects to one elem.
    if (K.Object.isNotNull(elem.nate))
    {
      throw 'Provided elem already has Nate object';
    }

    // Remove from current element is not supported.
    if (K.Object.isNotNull(this.elem))
    {
      throw 'This nate object is already attached to element.';
    }

    // Attach this object to elem.
    this.elem      = elem;
    this.elem.nate = this;

    for (let child in this.children)
    {
      throw 'This needs to be updated...';

      this.elem.appendChild(child);
    }
  }

  getElem()
  {
    this._confirmIsNotDeleted();

    return this.elem;
  }
}

}).call(this);
