K = window.K

class K.NateHtmlDom extends K.NateAbstractDom
  @ON_CLICK_EVENT_BLOCKER: () ->
    event = arguments[0] || window.event

    if event.shiftKey or event.ctrlKey
      # (SHIFT + click) or (CTRL + click) - don't block.
      rv = true
    else
      rv = (event.button != 0)
      event.returnValue = rv

    return rv

  constructor: (parentOrElem) ->
    # Hack to fix problem with extending ES class in coffee.
    @_init(parentOrElem)

    # Check the type of parentOrElem.
    isInstanceOfHtmlDom = (parentOrElem instanceof K.NateHtmlDom);
    if K.NateSvgDom? # K.NateSvgDom may be not included in the build
      isInstanceOfSvgDom  = (parentOrElem instanceof K.NateSvgDom)
    isElemOrText        = K.Elem.isElem(parentOrElem) or K.Elem.isText(parentOrElem)

    if (not isInstanceOfHtmlDom) and (not isElemOrText) and (not isInstanceOfSvgDom)
      throw 'parentOrElem should be Nate object or html-element'

  newNate: (elem = null) ->
    return new K.NateHtmlDom(elem)

  newChildFromElement: (elem, text) ->
    newNate = @newNate(elem)
    @addNateChild(newNate)
    if text?
      newNate.set('innerHTML', text)
    return newNate

  newTextNode: (text = '') ->
    textNode = document.createTextNode(text)
    return @newChildFromElement(textNode)

  newTxt: (theTxt) ->
    return @newTextNode(theTxt)

  newSpan: (text = null) ->
    span = document.createElement('span')
    return @newChildFromElement(span, text)

  newDiv: (text = null) ->
    div = document.createElement('div')
    return @newChildFromElement(div, text)

  newA: (text = null) ->
    a = document.createElement('a')
    return @newChildFromElement(a, text)

  newB: (text = null) ->
    a = document.createElement('b')
    return @newChildFromElement(a, text)

  newH1: (text = null) ->
    a = document.createElement('h1')
    return @newChildFromElement(a, text)

  newH2: (text = null) ->
    a = document.createElement('h2')
    return @newChildFromElement(a, text)

  newH3: (text = null) ->
    a = document.createElement('h3')
    return @newChildFromElement(a, text)

  newImg: (text = null) ->
    a = document.createElement('img')
    return @newChildFromElement(a, text)

  newLabel: (text = null) ->
    a = document.createElement('label')
    return @newChildFromElement(a, text)

  newP: (text = null) ->
    a = document.createElement('p')
    return @newChildFromElement(a, text)

  newPre: (text = null) ->
    a = document.createElement('pre')
    return @newChildFromElement(a, text)

  newUl: (text = null) ->
    a = document.createElement('ul')
    return @newChildFromElement(a, text)

  newLi: (text = null) ->
    a = document.createElement('Li')
    return @newChildFromElement(a, text)

  #
  # HTML Table
  #
  newTable: (text = null) ->
    a = document.createElement('table')
    return @newChildFromElement(a, text)

  newTBody: (text = null) ->
    a = document.createElement('tbody')
    return @newChildFromElement(a, text)

  newTr: (text = null) ->
    a = document.createElement('tr')
    return @newChildFromElement(a, text)

  newTd: (text = null) ->
    a = document.createElement('td')
    return @newChildFromElement(a, text)

  newTh: (text = null) ->
    a = document.createElement('th')
    return @newChildFromElement(a, text)

  #
  # HTML Inputs
  #
  newTextArea: () ->
    textarea = document.createElement('textarea')
    return @newChildFromElement(textarea)

  newOption: (text = null) ->
    option = document.createElement('option')
    return @newChildFromElement(option, text)

  newSelect: () ->
    return @newInputSelect()

  newInputCheckbox: () ->
    input = document.createElement('input')
    input.type = "checkbox"
    return @newChildFromElement(input)

  newInputText: () ->
    input = document.createElement('input')
    input.type = "text"
    return @newChildFromElement(input)

  newInputPassword: () ->
    input = document.createElement('input')
    input.type = "password"
    return @newChildFromElement(input)

  newInputRadio: () ->
    input = document.createElement('input')
    input.type = "radio"
    return @newChildFromElement(input)

  newInputSelect: () ->
    input = document.createElement('select')
    return @newChildFromElement(input)

  #
  # SVG, canvas etc.
  #
  newCanvas: (text = null) ->
    a = document.createElement('canvas')
    return @newChildFromElement(a, text)

  newSvg: () ->
    a = document.createElementNS('http://www.w3.org/2000/svg', 'svg')
    return @newChildFromElement(a)

  newSvgNate: () ->
    svgElem = document.createElementNS('http://www.w3.org/2000/svg', 'svg')
    svgNate = new K.NateSvgDom(svgElem)
    @addNateChild(svgNate)
    return svgNate

  #
  # nate methods
  #
  get$: () ->
    @_confirmIsNotDeleted()
    return $(@elem)

  setText: (txt) ->
    @set('innerHTML', txt)
    return @

  setOnClickEventBlocker: () ->
    @set('onclick', K.NateHtmlDom.ON_CLICK_EVENT_BLOCKER)
    return @

  set: (param, value) ->
    @_confirmIsNotDeleted()
    switch param
      when 'rect'
        value.applyToHtmlElement(@elem)
      when 'checked', 'href', 'id', 'innerHTML', 'value', \
      'selected', 'onclick', 'src', 'rowSpan', 'colSpan', 'target'
        K.updateProperty(@elem, param, value)
      when 'for' # 'for' in html  maps to 'htmlFor' in js
        K.updateProperty(@elem, 'htmlFor', value)
      when 'style'
        K.updateProperty(@elem.style, 'cssText', value)
      when 'cssText', 'cursor', \
      'display', 'float', \
      'height', 'width', 'left', 'top', 'right', 'bottom',\
      'margin', 'padding', \
      'overflow', 'overflowX', 'overflowY', \
      'position', 'visibility', 'zIndex', \
      'textAlign', 'verticalAlign', \
      'backgroundColor', 'color', \
      'border', 'borderCollapse', \
      'font', 'fontWeight'
        K.updateProperty(@elem.style, param, value)
      when 'on'
        # TODO: handling of other things than 'block'
        if value
          @set 'display', 'block'
        else
          @set 'display', 'none'
      when 'class'
        K.updateProperty(@elem, 'className', value)
      when 'cols', 'name', 'rows', 'wrap'
        @elem.setAttribute(param, value)
      else
        throw 'No handling for given property implemented: ' + param
    return @

  setData: (key, value) ->
    @_confirmIsNotDeleted()
    @elem.setAttribute('data-' + key, value)
    return @

  _decodeHtmlEntitiesToPlainText: (html) ->
    # Example:
    #   input  : Median &amp; average
    #   output : Median & average
    tempDivElement = document.createElement('div')
    tempDivElement.innerHTML = html

    if tempDivElement.childNodes.length is 0
      rv = ""
    else
      rv = tempDivElement.childNodes[0].nodeValue
    return rv

class K.NateHtmlDomHead extends K.NateHtmlDom
  setTitle: (title) ->
    # We decode title to plain text, because document.title works in different way
    # than static <title> tag. Browser's side title needs plain, decoded text
    # instead of html.
    document.title = @_decodeHtmlEntitiesToPlainText(title)

  setDocumentTitle: (title) ->
    @setTitle(title)
