const K = require('kcore');
require('./NateAbstractStatic');

K.NateSvgStatic = class K_NateSvgStatic extends K.NateAbstract
{
  constructor(parent)
  {
    super(parent);

    if (K.Object.isNotNull(parent))
    {
      parent.push(this);
    }

    this.attrs            = {};
    this.enabled          = true;
    this.tagOpenCloseMode = false;
  }

  renderAttributes()
  {
    let rv = '';

    for (let key in this.attrs)
    {
      rv += ' ' + key + '="' + this.attrs[key] + '"';
    }

    return rv;
  }

  setTag(tag, tagOpenCloseMode)
  {
    this.tag              = tag;
    this.tagOpenCloseMode = tagOpenCloseMode;
  }

  setAttributes(attrs)
  {
    this.attrs = attrs;
  }

  set(key, value)
  {
    this.attrs[key] = value;

    return this;
  }

  render()
  {
    let rv = '';

    if (this.enabled)
    {
      const tagName = K.Object.isNotNull(this.tag) ? this.tag : 'svg';

      // Open tag.
      rv = '<' + tagName + this.renderAttributes();

      // Close tag.
      if (this.tagOpenCloseMode)
      {
        // Just open-and-close at once tag.
        rv += ' />';
      }
      else
      {
        // Complex tag with possible children.
        rv += '>';
        rv += super.render();
        rv += '</' + this.tag + '>';
      }
    }

    return rv;
  }

  newSVGElement(tag, attrs = {}, tagOpenCloseMode = true)
  {
    const rv = new K.NateSvgStatic(this);
    rv.setTag(tag, attrs, tagOpenCloseMode);
    rv.setAttributes(attrs);
    return rv;
  }

  newCircle(attr)
  {
    return this.newSVGElement('circle', attr);
  }

  newPath(attr)
  {
    return this.newSVGElement('path', attr);
  }

  newLine(attr)
  {
    return this.newSVGElement('line', attr);
  }

  newPolyLine(attr)
  {
    return this.newSVGElement('polyline', attr);
  }

  clear()
  {
    // # TODO: implement this, but at the moment we need it only in K.NateXxxDom.
  }
}
